﻿using Hotline.Orders;
using Hotline.Push.FWMessage;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Share.Dtos.Push;
using Hotline.Share.Enums.Order;
using Hotline.Share.Tools;
using Microsoft.Extensions.Logging;
using SqlSugar;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Repository.SqlSugar.Orders;
public class OrderVisitRepository : BaseRepository<OrderVisit>, IOrderVisitRepository, IScopeDependency
{
    private readonly IRepository<OrderVisitDetail> _orderVisitDetailRepository;
    private readonly ILogger<OrderVisitRepository> _logger;
    private readonly IRepository<Order> _orderRepository;

    public OrderVisitRepository(ISugarUnitOfWork<HotlineDbContext> uow, IDataPermissionFilterBuilder dataPermissionFilterBuilder, IRepository<OrderVisitDetail> orderVisitDetailRepository, ILogger<OrderVisitRepository> logger, IRepository<Order> orderRepository) : base(uow, dataPermissionFilterBuilder)
    {
        _orderVisitDetailRepository = orderVisitDetailRepository;
        _logger = logger;
        _orderRepository = orderRepository;
    }

    /// <summary>
    /// 用户回访短信回复更新回访状态
    /// </summary>
    /// <param name="dto"></param>
    /// <param name="data"></param>
    /// <returns></returns>
    public async Task UpdateSmsReplyAsync(PushReceiveMessageDto dto, Message data)
    {
        if (dto.IsSmsReply == false || dto.SmsReplyContent.IsNullOrEmpty() || dto.ExternalId.IsNullOrEmpty()) return;

        var orderVisit = await GetAsync(dto.ExternalId)
             ?? throw new UserFriendlyException($"回访单不存在, visitId: {dto.ExternalId} message: {data.ToJson()}");
       
        Dictionary<string, string> dics = new()
        {
            { "1", $"非常满意|{EVisitState.Visited}|{ESeatEvaluate.VerySatisfied}|{EVoiceEvaluate.VerySatisfied}" },
            { "2", $"满意|{EVisitState.Visited}|{ESeatEvaluate.Satisfied}|{EVoiceEvaluate.Satisfied}"},
            { "3", $"一般|{EVisitState.Visited}|{ESeatEvaluate.Satisfied}|{EVoiceEvaluate.Normal}"},
            { "4", $"不满意|{EVisitState.SMSUnsatisfied}|{ESeatEvaluate.NoSatisfied}|{EVoiceEvaluate.NoSatisfied}"},
            { "5", $"非常不满意|{EVisitState.SMSUnsatisfied}|{ESeatEvaluate.NoSatisfied}|{EVoiceEvaluate.VeryNoSatisfied}"},
        };

        var replyTxt = dto.SmsReplyContent.Trim();
        var result = dics[replyTxt];
        if (result.IsNullOrEmpty()) throw new UserFriendlyException($"用户回复短信内容异常; reply: {replyTxt}");
        var replySplit = result.Split("|");
        if (new string[] { "4", "5" }.Contains(replyTxt))
        {
            // “短信不满意待回访”状态下，由其他方式再次进行回访，回访方式需更新为最新的回访方式
            // 故在此至为空
            orderVisit.VisitType = null;
        }
        orderVisit.NowEvaluate = new Share.Dtos.Kv(replyTxt, replySplit[0]);

        orderVisit.VisitTime = DateTime.Now;
        orderVisit.VisitState = replySplit[1].ToEnum<EVisitState>();
        await UpdateAsync(orderVisit, ignoreNullColumns: false);

        if (orderVisit.VisitState == EVisitState.Visited)
        {
            var order = await _orderRepository.GetAsync(orderVisit.OrderId);
            if (order is not null)
            {
                order.Visited(replyTxt, replySplit[0]);
                await _orderRepository.UpdateAsync(order);
            }
        }
        var detailOrg = await _orderVisitDetailRepository.Queryable()
            .Where(m => m.VisitId == orderVisit.Id && m.VisitTarget == EVisitTarget.Org).FirstAsync();
        if (detailOrg is not null)
        {
            detailOrg.OrgProcessingResults = new Share.Dtos.Kv(replyTxt, replySplit[0]);
            detailOrg.OrgHandledAttitude = new Share.Dtos.Kv(replyTxt, replySplit[0]);
            await _orderVisitDetailRepository.UpdateAsync(detailOrg);
        }

        var detailSeat = await _orderVisitDetailRepository.Queryable()
           .Where(m => m.VisitId == orderVisit.Id && m.VisitTarget == EVisitTarget.Seat).FirstAsync();
        if (detailSeat is not null)
        {
            detailSeat.SeatEvaluate ??= replySplit[2].ToEnum<ESeatEvaluate>();
            detailSeat.VoiceEvaluate ??= replySplit[3].ToEnum<EVoiceEvaluate>();

            await _orderVisitDetailRepository.UpdateAsync(detailSeat);
        }
    }
}
